@@ -90,7 +90,7 @@ test_that("ard_categorical() univariate & specified denomiator", {
       dplyr::filter(stat_name %in% "p") |>
       dplyr::pull(stat) |>
       as.numeric(),
-      table(mtcars$am) |> prop.table() |> as.numeric() %>% `/`(100) # styler: off
+    table(mtcars$am) |> prop.table() |> as.numeric() %>% `/`(100) # styler: off
   )
 
   expect_equal(
@@ -310,7 +310,7 @@ test_that("ard_categorical(denominator='cell') works", {
     ard_categorical(
       data = df_missing,
       variables = c(all_na_lgl, all_na_fct),
-      statistic = ~ categorical_summary_fns(c("n", "N")),
+      statistic = ~ c("n", "N"),
       denominator = "cell"
     ) |>
       dplyr::pull(stat) |>
@@ -323,7 +323,7 @@ test_that("ard_categorical(denominator='cell') works", {
       data = df_missing,
       variables = c(all_na_lgl, all_na_fct),
       by = letters,
-      statistic = ~ categorical_summary_fns(c("n", "N")),
+      statistic = ~ c("n", "N"),
       denominator = "cell"
     ) |>
       dplyr::pull(stat) |>
@@ -382,7 +382,7 @@ test_that("ard_categorical(denominator='row') works", {
         ADSL,
         variables = "AGEGR1", by = "ARM",
         denominator = "row",
-        statistic = list(AGEGR1 = categorical_summary_fns(c("n", "N"))),
+        statistic = list(AGEGR1 = c("n", "N")),
         fmt_fn = list(AGEGR1 = list("n" = 2))
       ),
     NA
@@ -401,7 +401,7 @@ test_that("ard_categorical(denominator='row') works", {
     ard_categorical(
       data = df_missing,
       variable = all_na_lgl,
-      statistic = ~ categorical_summary_fns(c("n", "N")),
+      statistic = ~ c("n", "N"),
       denominator = "row"
     ) |>
       dplyr::pull(stat) |>
@@ -414,7 +414,7 @@ test_that("ard_categorical(denominator='row') works", {
       data = df_missing,
       variable = all_na_lgl,
       by = letters,
-      statistic = ~ categorical_summary_fns(c("n", "N")),
+      statistic = ~ c("n", "N"),
       denominator = "row"
     ) |>
       dplyr::pull(stat) |>
@@ -437,7 +437,7 @@ test_that("ard_categorical(denominator='column') works", {
     ard_categorical(
       data = df_missing,
       variable = all_na_lgl,
-      statistic = ~ categorical_summary_fns(c("n", "N")),
+      statistic = ~ c("n", "N"),
       denominator = "column"
     ) |>
       dplyr::pull(stat) |>
@@ -450,7 +450,7 @@ test_that("ard_categorical(denominator='column') works", {
       data = df_missing,
       variable = all_na_lgl,
       by = letters,
-      statistic = ~ categorical_summary_fns(c("n", "N")),
+      statistic = ~ c("n", "N"),
       denominator = "column"
     ) |>
       dplyr::pull(stat) |>
@@ -464,7 +464,7 @@ test_that("ard_categorical(denominator='column') works", {
     ard_categorical(
       data = df_missing,
       variable = all_na_lgl,
-      statistic = ~ categorical_summary_fns(c("n", "N")),
+      statistic = ~ c("n", "N"),
       denominator = "column"
     ) |>
       dplyr::pull(stat) |>
@@ -477,7 +477,7 @@ test_that("ard_categorical(denominator='column') works", {
       data = df_missing,
       variable = all_na_lgl,
       by = letters,
-      statistic = ~ categorical_summary_fns(c("n", "N")),
+      statistic = ~ c("n", "N"),
       denominator = "column"
     ) |>
       dplyr::pull(stat) |>
@@ -546,7 +546,7 @@ test_that("ard_categorical(denominator=<data frame without counts>) works", {
       dplyr::mutate(AGEGR1 = NA) |>
       ard_categorical(
         variables = AGEGR1,
-        statistic = ~ categorical_summary_fns(c("n", "p")),
+        statistic = ~ c("n", "p"),
         denominator = rep_len(list(ADSL), 10L) |> dplyr::bind_rows()
       ) |>
       dplyr::pull(stat) |>
@@ -562,7 +562,7 @@ test_that("ard_categorical(denominator=<data frame without counts>) works", {
       ard_categorical(
         variables = AGEGR1,
         by = ARM,
-        statistic = ~ categorical_summary_fns(c("n", "p")),
+        statistic = ~ c("n", "p"),
         denominator = rep_len(list(ADSL), 10L) |> dplyr::bind_rows()
       ) |>
       dplyr::pull(stat) |>
@@ -572,51 +572,6 @@ test_that("ard_categorical(denominator=<data frame without counts>) works", {
   )
 })
 
-test_that("ard_categorical(statistic) works with custom fns", {
-  expect_snapshot(
-    ard_custom_fns <-
-      ard_categorical(
-        ADSL,
-        variables = AGEGR1,
-        statistic =
-          ~ categorical_summary_fns(
-            other_stats = list(
-              mode = function(x) {
-                table(x) |>
-                  sort(decreasing = TRUE) |>
-                  names() |>
-                  getElement(1)
-              },
-              length = function(x) length(x)
-            )
-          )
-      )
-  )
-
-  expect_equal(
-    ard_custom_fns |>
-      dplyr::select(-variable_level) |>
-      dplyr::filter(stat_name %in% c("mode", "length")),
-    ard_categorical(
-      ADSL,
-      variables = AGEGR1,
-      statistic =
-        ~ categorical_summary_fns(
-          summaries = list(),
-          other_stats = list(
-            mode = function(x) {
-              table(x) |>
-                sort(decreasing = TRUE) |>
-                names() |>
-                getElement(1)
-            },
-            length = function(x) length(x)
-          )
-        )
-    )
-  )
-})
-
 test_that("ard_categorical() and ARD column names", {
   ard_colnames <- c(
     "group1", "group1_level", "variable", "variable_level",
@@ -681,3 +636,195 @@ test_that("ard_categorical() and all NA columns", {
       ard_categorical(variables = AGEGR1)
   )
 })
+
+test_that("ard_categorical() can handle non-syntactic column names", {
+  expect_equal(
+    ADSL |>
+      dplyr::mutate(`Age Group` = AGEGR1) |>
+      ard_categorical(variables = `Age Group`) |>
+      dplyr::select(stat),
+    ADSL |>
+      ard_categorical(variables = AGEGR1) |>
+      dplyr::select(stat)
+  )
+
+  expect_equal(
+    ADSL |>
+      dplyr::mutate(`Age Group` = AGEGR1) |>
+      ard_categorical(variables = "Age Group") |>
+      dplyr::select(stat, error),
+    ADSL |>
+      ard_categorical(variables = AGEGR1) |>
+      dplyr::select(stat, error)
+  )
+
+  expect_equal(
+    ADSL |>
+      dplyr::mutate(`Arm Var` = ARM, `Age Group` = AGEGR1) |>
+      ard_categorical(by = `Arm Var`, variables = "Age Group") |>
+      dplyr::select(stat, error),
+    ADSL |>
+      ard_categorical(by = ARM, variables = AGEGR1) |>
+      dplyr::select(stat, error)
+  )
+
+
+  expect_equal(
+    ADSL |>
+      dplyr::mutate(`Arm Var` = ARM, `Age Group` = AGEGR1) |>
+      ard_categorical(strata = "Arm Var", variables = `Age Group`) |>
+      dplyr::select(stat, error),
+    ADSL |>
+      ard_categorical(strata = ARM, variables = AGEGR1) |>
+      dplyr::select(stat, error)
+  )
+})
+
+test_that("ard_categorical(strata) returns results in proper order", {
+  expect_equal(
+    ard_categorical(
+      ADAE |>
+        dplyr::arrange(AESEV != "SEVERE") |> # put SEVERE at the top
+        dplyr::mutate(AESEV = factor(AESEV, levels = c("MILD", "MODERATE", "SEVERE"))) |>
+        dplyr::mutate(ANY_AE = 1L),
+      by = TRTA,
+      strata = AESEV,
+      variables = ANY_AE,
+      denominator = ADSL |> dplyr::rename(TRTA = ARM)
+    ) |>
+      dplyr::select(group2_level) |>
+      unlist() |>
+      unique() |>
+      as.character(),
+    c("MILD", "MODERATE", "SEVERE")
+  )
+})
+
+test_that("ard_categorical(by) messages about protected names", {
+  mtcars2 <- mtcars |>
+    dplyr::mutate(
+      variable = am,
+      variable_level = cyl,
+      by = am,
+      by_level = cyl
+    )
+
+  expect_snapshot(
+    error = TRUE,
+    ard_categorical(mtcars2, by = variable, variables = gear)
+  )
+
+  expect_error(
+    ard_categorical(mtcars2, by = variable_level, variables = gear),
+    'The `by` argument cannot include variables named "variable" and "variable_level".'
+  )
+})
+
+# - test if function parameters can be used as variable names without error
+test_that("ard_categorical() works when using generic names ", {
+  # rename some variables
+  mtcars2 <- mtcars %>%
+    dplyr::rename("variable" = am, "variable_level" = cyl, "by" = disp, "group1_level" = gear)
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, cyl), by = disp, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(variable, variable_level), by = by, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(cyl, am), by = gear, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(variable_level, variable), by = group1_level, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(gear, am), by = disp, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(group1_level, variable), by = by, denominator = "row") |> dplyr::select(stat)
+  )
+
+  # rename vars
+  mtcars2 <- mtcars %>%
+    dplyr::rename("N" = am, "p" = cyl, "name" = disp, "group1_level" = gear)
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, cyl), by = disp, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(N, p), by = name, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(disp, gear), by = am, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(name, group1_level), by = N, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, disp), by = gear, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(N, name), by = group1_level, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, disp), by = cyl, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(N, name), by = p, denominator = "row") |> dplyr::select(stat)
+  )
+
+  # rename vars
+  mtcars2 <- mtcars %>%
+    dplyr::rename("n" = am, "mean" = cyl, "p.std.error" = disp, "n_unweighted" = gear)
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(gear, cyl), by = disp, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(n_unweighted, mean), by = p.std.error, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(gear, cyl), by = am, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(n_unweighted, mean), by = n, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, disp), by = cyl, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(n, p.std.error), by = mean, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, disp), by = gear, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(n, p.std.error), by = n_unweighted, denominator = "row") |> dplyr::select(stat)
+  )
+
+  # rename vars
+  mtcars2 <- mtcars %>%
+    dplyr::rename("N_unweighted" = am, "p_unweighted" = cyl, "column" = disp, "row" = gear)
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, cyl), by = disp, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(N_unweighted, p_unweighted), by = column, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(disp, gear), by = am, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(column, row), by = N_unweighted, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, disp), by = cyl, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(N_unweighted, column), by = p_unweighted, denominator = "row") |> dplyr::select(stat)
+  )
+
+  expect_equal(
+    ard_categorical(mtcars, variables = c(am, disp), by = gear, denominator = "row") |> dplyr::select(stat),
+    ard_categorical(mtcars2, variables = c(N_unweighted, column), by = row, denominator = "row") |> dplyr::select(stat)
+  )
+})
+
+test_that("ard_categorical(by) messages about protected names", {
+  mtcars2 <- mtcars %>%
+    dplyr::rename("variable" = am, "variable_level" = cyl, "by" = disp, "group1_level" = gear)
+
+  expect_snapshot(
+    error = TRUE,
+    ard_categorical(mtcars2, by = variable, variables = by)
+  )
+
+  expect_error(
+    ard_categorical(mtcars2, by = variable_level, variables = by),
+    'The `by` argument cannot include variables named "variable" and "variable_level".'
+  )
+})
