@@ -3,7 +3,7 @@ test_that("ard_stack() works", {
   expect_error(
     ard1 <- ard_stack(
       data = mtcars,
-      by = "cyl",
+      .by = "cyl",
       ard_continuous(variables = "mpg"),
       ard_dichotomous(variables = "vs")
     ),
@@ -15,7 +15,7 @@ test_that("ard_stack() works", {
     bind_ard(
       ard_continuous(data = mtcars, by = "cyl", variables = "mpg"),
       ard_dichotomous(data = mtcars, by = "cyl", variables = "vs"),
-      ard_categorical(data = mtcars, variables = "cyl", statistic = everything() ~ categorical_summary_fns("N")),
+      ard_categorical(data = mtcars, variables = "cyl"),
       .order = TRUE
     ),
     ignore_function_env = TRUE
@@ -26,7 +26,7 @@ test_that("ard_stack() works", {
     ard1,
     ard_stack(
       data = mtcars,
-      by = cyl,
+      .by = cyl,
       ard_continuous(variables = mpg),
       ard_dichotomous(variables = vs)
     ),
@@ -42,7 +42,7 @@ test_that("ard_stack() works", {
     ard1,
     ard_stack(
       data = mtcars2,
-      by = all_of(by),
+      .by = all_of(by),
       ard_continuous(variables = all_of(var_cont)),
       ard_dichotomous(variables = all_of(var_cat))
     )
@@ -52,7 +52,7 @@ test_that("ard_stack() works", {
   expect_error(
     ard2 <- ard_stack(
       data = mtcars,
-      by = NULL,
+      .by = NULL,
       ard_continuous(variables = "mpg"),
       ard_dichotomous(variables = "vs")
     ),
@@ -74,7 +74,6 @@ test_that("ard_stack() works", {
     ard2,
     ard_stack(
       data = mtcars2,
-      by = NULL,
       ard_continuous(variables = all_of(var_cont)),
       ard_dichotomous(variables = all_of(var_cat))
     )
@@ -85,7 +84,7 @@ test_that("ard_stack() adding overalls", {
   expect_error(
     ard_test <- ard_stack(
       data = mtcars,
-      by = "cyl",
+      .by = "cyl",
       ard_continuous(variables = "mpg"),
       ard_dichotomous(variables = "vs"),
       .overall = TRUE
@@ -99,7 +98,7 @@ test_that("ard_stack() adding overalls", {
     bind_ard(
       ard_continuous(data = mtcars, by = "cyl", variables = "mpg"),
       ard_dichotomous(data = mtcars, by = "cyl", variables = "vs"),
-      ard_categorical(data = mtcars, variables = "cyl", statistic = everything() ~ categorical_summary_fns("N")),
+      ard_categorical(data = mtcars, variables = "cyl"),
       ard_continuous(data = mtcars, variables = "mpg"),
       ard_dichotomous(data = mtcars, variables = "vs"),
       .update = TRUE,
@@ -113,7 +112,7 @@ test_that("ard_stack() adding missing/attributes", {
   expect_error(
     ard_test <- ard_stack(
       data = mtcars,
-      by = "cyl",
+      .by = "cyl",
       ard_continuous(variables = "mpg"),
       ard_dichotomous(variables = "vs"),
       .missing = TRUE,
@@ -127,9 +126,39 @@ test_that("ard_stack() adding missing/attributes", {
     bind_ard(
       ard_continuous(data = mtcars, by = "cyl", variables = "mpg"),
       ard_dichotomous(data = mtcars, by = "cyl", variables = "vs"),
-      ard_categorical(data = mtcars, variables = "cyl", statistic = everything() ~ categorical_summary_fns("N")),
-      ard_missing(data = mtcars, variables = c("cyl", "mpg", "vs")),
-      ard_attributes(data = mtcars, variables = c("cyl", "mpg", "vs")),
+      ard_categorical(data = mtcars, variables = "cyl"),
+      ard_missing(data = mtcars, by = "cyl", variables = c("mpg", "vs")),
+      ard_attributes(mtcars, variables = c("cyl", "mpg", "vs")),
+      .update = TRUE,
+      .order = TRUE
+    )
+  )
+
+  # including `.overall=TRUE`
+  expect_error(
+    ard_test_overall <- ard_stack(
+      data = mtcars,
+      .by = "cyl",
+      ard_continuous(variables = "mpg"),
+      ard_dichotomous(variables = "vs"),
+      .missing = TRUE,
+      .overall = TRUE,
+      .attributes = TRUE
+    ),
+    NA
+  )
+
+  expect_equal(
+    ard_test_overall,
+    bind_ard(
+      ard_continuous(data = mtcars, by = "cyl", variables = "mpg"),
+      ard_dichotomous(data = mtcars, by = "cyl", variables = "vs"),
+      ard_missing(data = mtcars, by = "cyl", variables = c("mpg", "vs")),
+      ard_continuous(data = mtcars, variables = "mpg"),
+      ard_dichotomous(data = mtcars, variables = "vs"),
+      ard_missing(data = mtcars, variables = c("mpg", "vs")),
+      ard_categorical(data = mtcars, variables = "cyl"),
+      ard_attributes(mtcars, variables = c("cyl", "mpg", "vs")),
       .update = TRUE,
       .order = TRUE
     )
@@ -141,7 +170,7 @@ test_that("ard_stack() .shuffle argument", {
   expect_error(
     ard_test <- ard_stack(
       data = mtcars,
-      by = "cyl",
+      .by = "cyl",
       ard_continuous(variables = "mpg"),
       ard_dichotomous(variables = "vs"),
       .shuffle = TRUE
@@ -154,9 +183,68 @@ test_that("ard_stack() .shuffle argument", {
     bind_ard(
       ard_continuous(data = mtcars, by = "cyl", variables = "mpg"),
       ard_dichotomous(data = mtcars, by = "cyl", variables = "vs"),
-      ard_categorical(data = mtcars, variables = "cyl", statistic = everything() ~ categorical_summary_fns("N")),
+      ard_categorical(data = mtcars, variables = "cyl"),
       .order = TRUE
     ) |>
       shuffle_ard()
   )
 })
+
+test_that("ard_stack() adding total N", {
+  expect_equal(
+    ard_stack(
+      mtcars,
+      .by = am,
+      ard_continuous(variables = mpg),
+      .total_n = TRUE
+    ) |>
+      tail(n = 1) |>
+      dplyr::select(-all_ard_groups(), -all_ard_variables("levels")),
+    ard_total_n(mtcars)
+  )
+})
+
+test_that("ard_stack() works with namespaced functions", {
+  expect_equal(
+    ard_stack(
+      data = mtcars,
+      cards::ard_continuous(variables = "mpg")
+    ),
+    ard_stack(
+      data = mtcars,
+      ard_continuous(variables = "mpg")
+    )
+  )
+})
+
+test_that("ard_stack() messaging", {
+  expect_snapshot(
+    ard_stack(
+      data = mtcars,
+      ard_continuous(variables = "mpg"),
+      .overall = TRUE
+    ) |>
+      head(1L)
+  )
+
+  # by argument doesn't include period in front
+  expect_snapshot(
+    error = TRUE,
+    ard_stack(ADSL, by = "ARM", ard_continuous(variables = AGE))
+  )
+})
+
+test_that("ard_stack() complex call error", {
+  expect_snapshot(
+    {
+      complex_call <- list()
+      complex_call$ard_continuous <- ard_continuous
+      ard_stack(
+        data = mtcars,
+        .by = am,
+        complex_call$ard_continuous(variables = "mpg"),
+      )
+    },
+    error = TRUE
+  )
+})
